@@ -84,7 +84,7 @@ int callback_auth_basic (const struct _u_request * request, struct _u_response *
   return U_CALLBACK_CONTINUE;
 }
 
-#ifndef U_DISABLE_WEBSOCKET
+#ifndef U_DISABLE_GNUTLS
 /**
  * Callback function on client certificate authentication
  */
@@ -133,13 +133,13 @@ int main (int argc, char **argv) {
   ulfius_add_endpoint_by_val(&instance, "GET", PREFIX, "/basic", 1, &callback_auth_basic, NULL);
   ulfius_add_endpoint_by_val(&instance, "GET", PREFIX, "/default", 1, &callback_auth_basic, NULL);
   ulfius_add_endpoint_by_val(&instance, "GET", PREFIX, "/default", 0, &callback_auth_basic_body, NULL);
-#ifndef U_DISABLE_WEBSOCKET
+#ifndef U_DISABLE_GNUTLS
   if (argc > 3) {
     ulfius_add_endpoint_by_val(&instance, "GET", PREFIX, "/client_cert", 0, &callback_auth_client_cert, NULL);
   }
 #endif
   
-#ifndef U_DISABLE_WEBSOCKET
+#ifndef U_DISABLE_GNUTLS
   // Start the framework
   if (argc > 3) {
     char * server_key = read_file(argv[1]), * server_pem = read_file(argv[2]), * root_ca_pem = read_file(argv[3]);
@@ -162,7 +162,7 @@ int main (int argc, char **argv) {
     // Wait for the user to press <enter> on the console to quit the application
     printf("Press <enter> to quit server\n");
     getchar();
-#ifndef U_DISABLE_WEBSOCKET
+#ifndef U_DISABLE_GNUTLS
   } else {
     printf("Error starting framework\n");
   }
