@@ -11,6 +11,7 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <netinet/tcp.h>
+#include <ifaddrs.h>
 
 #ifndef _WIN32
   #include <sys/socket.h>
@@ -49,6 +50,16 @@
 #define SUBJECT "subject"
 #define BODY "mail body"
 
+#define KEY1 "key1"
+#define KEY2 "kéy2"
+#define KEY3 "key 3"
+#define KEY4 "key%204"
+
+#define VALUE1 "value1"
+#define VALUE2 "value2:'with$%stuff"
+#define VALUE3 "value 3"
+#define VALUE4 "valué(4)"
+
 struct smtp_manager {
   char * mail_data;
   unsigned int port;
@@ -268,37 +279,22 @@ static char * get_file_content(const char * file_path) {
   return buffer;
 }
 
-/**
- * decode a u_map into a string
- */
-char * print_map(const struct _u_map * map) {
-  char * line, * to_return = NULL;
-  const char **keys, * value;
-  int len, i;
-  if (map != NULL) {
-    keys = u_map_enum_keys(map);
-    for (i=0; keys[i] != NULL; i++) {
-      value = u_map_get(map, keys[i]);
-      len = snprintf(NULL, 0, "key is %s, value is %s", keys[i], value);
-      line = o_malloc((len+1)*sizeof(char));
-      snprintf(line, (len+1), "key is %s, value is %s", keys[i], value);
-      if (to_return != NULL) {
-        len = o_strlen(to_return) + o_strlen(line) + 1;
-        to_return = o_realloc(to_return, (len+1)*sizeof(char));
-        if (o_strlen(to_return) > 0) {
-          strcat(to_return, "\n");
-        }
-      } else {
-        to_return = o_malloc((o_strlen(line) + 1)*sizeof(char));
-        to_return[0] = 0;
-      }
-      strcat(to_return, line);
-      o_free(line);
-    }
-    return to_return;
-  } else {
-    return NULL;
+static int has_ipv6() {
+  struct ifaddrs * ifaddr, * ifa;
+  
+  if (getifaddrs(&ifaddr) == -1) {
+    return 0;
   }
+  for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+    if (ifa->ifa_addr != NULL) {
+      if (ifa->ifa_addr->sa_family == AF_INET6) {
+        freeifaddrs(ifaddr);
+        return 1;
+      }
+    }
+  }
+  freeifaddrs(ifaddr);
+  return 0;
 }
 
 int callback_function_empty(const struct _u_request * request, struct _u_response * response, void * user_data) {
@@ -522,6 +518,33 @@ int callback_function_shared_data_2(const struct _u_request * request, struct _u
   return U_CALLBACK_CONTINUE;
 }
 
+int callback_send_request_no_rest(const struct _u_request * request, struct _u_response * response, void * user_data) {
+  struct _u_request * request_orig = (struct _u_request *)user_data;
+  const char ** keys;
+  size_t i;
+
+  ck_assert_int_eq(u_map_count(request->map_url), u_map_count(request_orig->map_url));
+  keys = u_map_enum_keys(request->map_url);
+  for (i=0; keys[i]!=NULL; i++) {
+    if (o_strlen(u_map_get(request->map_url, keys[i]))) {
+      ck_assert_str_eq(u_map_get(request->map_url, keys[i]), u_map_get(request_orig->map_url, keys[i]));
+    }
+  }
+  response->status = 208;
+
+  return U_CALLBACK_CONTINUE;
+}
+
+int callback_send_request_rest(const struct _u_request * request, struct _u_response * response, void * user_data) {
+  ck_assert_int_eq(u_map_count(request->map_url), 3);
+  ck_assert_str_eq(VALUE1, u_map_get(request->map_url, KEY1));
+  ck_assert_str_eq(VALUE3, u_map_get(request->map_url, KEY2));
+  ck_assert_str_eq(VALUE4, u_map_get(request->map_url, KEY3));
+  response->status = 208;
+
+  return U_CALLBACK_CONTINUE;
+}
+
 int via_free_with_test = 0;
 
 void free_with_test(void * ptr) {
@@ -749,14 +772,16 @@ START_TEST(test_ulfius_net_type_endpoint)
   ulfius_clean_request(&request);
   ulfius_clean_response(&response);
   
-  ulfius_init_request(&request);
-  request.http_url = o_strdup("http://[::1]:8080/empty");
-  request.network_type = U_USE_IPV6;
-  ulfius_init_response(&response);
-  ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
-  ck_assert_int_eq(response.status, 200);
-  ulfius_clean_request(&request);
-  ulfius_clean_response(&response);
+  if (has_ipv6()) {
+    ulfius_init_request(&request);
+    request.http_url = o_strdup("http://[::1]:8080/empty");
+    request.network_type = U_USE_IPV6;
+    ulfius_init_response(&response);
+    ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
+    ck_assert_int_eq(response.status, 200);
+    ulfius_clean_request(&request);
+    ulfius_clean_response(&response);
+  }
   
   ulfius_stop_framework(&u_instance);
   ulfius_clean_instance(&u_instance);
@@ -772,14 +797,16 @@ START_TEST(test_ulfius_net_type_endpoint)
   ck_assert_int_ne(ulfius_send_http_request(&request, NULL), U_OK);
   ulfius_clean_request(&request);
   
-  ulfius_init_request(&request);
-  request.http_url = o_strdup("http://[::1]:8080/empty");
-  request.network_type = U_USE_IPV6;
-  ulfius_init_response(&response);
-  ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
-  ck_assert_int_eq(response.status, 200);
-  ulfius_clean_request(&request);
-  ulfius_clean_response(&response);
+  if (has_ipv6()) {
+    ulfius_init_request(&request);
+    request.http_url = o_strdup("http://[::1]:8080/empty");
+    request.network_type = U_USE_IPV6;
+    ulfius_init_response(&response);
+    ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
+    ck_assert_int_eq(response.status, 200);
+    ulfius_clean_request(&request);
+    ulfius_clean_response(&response);
+  }
   
   ulfius_stop_framework(&u_instance);
   ulfius_clean_instance(&u_instance);
@@ -798,11 +825,13 @@ START_TEST(test_ulfius_net_type_endpoint)
   ulfius_clean_request(&request);
   ulfius_clean_response(&response);
   
-  ulfius_init_request(&request);
-  request.http_url = o_strdup("http://[::1]:8080/empty");
-  request.network_type = U_USE_IPV6;
-  ck_assert_int_ne(ulfius_send_http_request(&request, NULL), U_OK);
-  ulfius_clean_request(&request);
+  if (has_ipv6()) {
+    ulfius_init_request(&request);
+    request.http_url = o_strdup("http://[::1]:8080/empty");
+    request.network_type = U_USE_IPV6;
+    ck_assert_int_ne(ulfius_send_http_request(&request, NULL), U_OK);
+    ulfius_clean_request(&request);
+  }
   
   ulfius_stop_framework(&u_instance);
   ulfius_clean_instance(&u_instance);
@@ -825,41 +854,45 @@ START_TEST(test_ulfius_net_type_endpoint)
   ulfius_clean_request(&request);
   ulfius_clean_response(&response);
   
-  ulfius_init_request(&request);
-  request.http_url = o_strdup("http://[::1]:8080/empty");
-  request.network_type = U_USE_IPV6;
-  ck_assert_int_ne(ulfius_send_http_request(&request, NULL), U_OK);
-  ulfius_clean_request(&request);
+  if (has_ipv6()) {
+    ulfius_init_request(&request);
+    request.http_url = o_strdup("http://[::1]:8080/empty");
+    request.network_type = U_USE_IPV6;
+    ck_assert_int_ne(ulfius_send_http_request(&request, NULL), U_OK);
+    ulfius_clean_request(&request);
+  }
   
   ulfius_stop_framework(&u_instance);
   ulfius_clean_instance(&u_instance);
   
-  // Test network binding to an IPV6 address
-  memset(&ipv6addr, 0, sizeof(ipv6addr));
-  ipv6addr.sin6_family = AF_INET6;
-  ipv6addr.sin6_port = htons(8080);
-  ipv6addr.sin6_addr = in6addr_loopback;
-  ck_assert_int_eq(ulfius_init_instance_ipv6(&u_instance, 8080, &ipv6addr, U_USE_IPV6, NULL), U_OK);
-  ck_assert_int_eq(ulfius_add_endpoint_by_val(&u_instance, "GET", "empty", NULL, 0, &callback_function_empty, NULL), U_OK);
-  ck_assert_int_eq(ulfius_start_framework(&u_instance), U_OK);
+  if (has_ipv6()) {
+    // Test network binding to an IPV6 address
+    memset(&ipv6addr, 0, sizeof(ipv6addr));
+    ipv6addr.sin6_family = AF_INET6;
+    ipv6addr.sin6_port = htons(8080);
+    ipv6addr.sin6_addr = in6addr_loopback;
+    ck_assert_int_eq(ulfius_init_instance_ipv6(&u_instance, 8080, &ipv6addr, U_USE_IPV6, NULL), U_OK);
+    ck_assert_int_eq(ulfius_add_endpoint_by_val(&u_instance, "GET", "empty", NULL, 0, &callback_function_empty, NULL), U_OK);
+    ck_assert_int_eq(ulfius_start_framework(&u_instance), U_OK);
 
-  ulfius_init_request(&request);
-  request.http_url = o_strdup("http://127.0.0.1:8080/empty");
-  request.network_type = U_USE_IPV4;
-  ck_assert_int_ne(ulfius_send_http_request(&request, NULL), U_OK);
-  ulfius_clean_request(&request);
+    ulfius_init_request(&request);
+    request.http_url = o_strdup("http://127.0.0.1:8080/empty");
+    request.network_type = U_USE_IPV4;
+    ck_assert_int_ne(ulfius_send_http_request(&request, NULL), U_OK);
+    ulfius_clean_request(&request);
   
-  ulfius_init_request(&request);
-  request.http_url = o_strdup("http://[::1]:8080/empty");
-  request.network_type = U_USE_IPV6;
-  ulfius_init_response(&response);
-  ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
-  ck_assert_int_eq(response.status, 200);
-  ulfius_clean_request(&request);
-  ulfius_clean_response(&response);
+    ulfius_init_request(&request);
+    request.http_url = o_strdup("http://[::1]:8080/empty");
+    request.network_type = U_USE_IPV6;
+    ulfius_init_response(&response);
+    ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
+    ck_assert_int_eq(response.status, 200);
+    ulfius_clean_request(&request);
+    ulfius_clean_response(&response);
   
-  ulfius_stop_framework(&u_instance);
-  ulfius_clean_instance(&u_instance);
+    ulfius_stop_framework(&u_instance);
+    ulfius_clean_instance(&u_instance);
+  }
 }
 END_TEST
 #endif
@@ -1379,7 +1412,6 @@ START_TEST(test_ulfius_shared_data)
 }
 END_TEST
 
-
 START_TEST(test_ulfius_malformed_requests)
 {
   struct _u_instance u_instance;
@@ -1442,6 +1474,44 @@ START_TEST(test_ulfius_malformed_requests)
 }
 END_TEST
 
+START_TEST(test_ulfius_send_http_request)
+{
+  struct _u_instance u_instance;
+  struct _u_request request;
+  struct _u_response response;
+  
+  ck_assert_int_eq(ulfius_init_request(&request), U_OK);
+  ck_assert_int_eq(ulfius_init_instance(&u_instance, 8080, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&u_instance, "GET", "/norest", "*", 0, &callback_send_request_no_rest, &request), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&u_instance, "GET", "/rest", "/:"KEY1"/:"KEY2"/:"KEY3"/", 0, &callback_send_request_rest, NULL), U_OK);
+  ck_assert_int_eq(ulfius_start_framework(&u_instance), U_OK);
+
+  ck_assert_int_eq(ulfius_init_response(&response), U_OK);
+  ck_assert_int_eq(ulfius_set_request_properties(&request, U_OPT_HTTP_URL, "http://localhost:8080/norest/test_request/give me some space/",
+                                                           U_OPT_URL_PARAMETER, KEY1, VALUE1,
+                                                           U_OPT_URL_PARAMETER, KEY2, VALUE2,
+                                                           U_OPT_URL_PARAMETER, KEY3, VALUE3,
+                                                           U_OPT_URL_PARAMETER, KEY4, NULL,
+                                                           U_OPT_NONE), U_OK);
+  ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
+  ck_assert_int_eq(208, response.status);
+  ulfius_clean_request(&request);
+  ulfius_clean_response(&response);
+
+  ck_assert_int_eq(ulfius_init_request(&request), U_OK);
+  ck_assert_int_eq(ulfius_init_response(&response), U_OK);
+  ck_assert_int_eq(ulfius_set_request_properties(&request, U_OPT_HTTP_URL, "http://localhost:8080/rest/" VALUE1 "/" VALUE3 "/" VALUE4,
+                                                           U_OPT_NONE), U_OK);
+  ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_OK);
+  ck_assert_int_eq(208, response.status);
+  ulfius_clean_request(&request);
+  ulfius_clean_response(&response);
+
+  ulfius_stop_framework(&u_instance);
+  ulfius_clean_instance(&u_instance);
+}
+END_TEST
+
 #ifndef U_DISABLE_GNUTLS
 START_TEST(test_ulfius_server_ca_trust)
 {
@@ -1569,6 +1639,7 @@ static Suite *ulfius_suite(void)
   tcase_add_test(tc_core, test_ulfius_follow_redirect);
   tcase_add_test(tc_core, test_ulfius_shared_data);
   tcase_add_test(tc_core, test_ulfius_malformed_requests);
+  tcase_add_test(tc_core, test_ulfius_send_http_request);
 #ifndef U_DISABLE_GNUTLS
   tcase_add_test(tc_core, test_ulfius_server_ca_trust);
   tcase_add_test(tc_core, test_ulfius_client_certificate);
